set echo on

CREATE OR REPLACE PACKAGE log_pkg AS

   FUNCTION start_log( 
            p_id  IN VARCHAR2,
            p_lvl IN PLS_INTEGER DEFAULT 1 
            ) RETURN VARCHAR2;

   PROCEDURE log( 
             p_id  IN VARCHAR2,
             p_msg IN VARCHAR2,
             p_lvl IN PLS_INTEGER DEFAULT 1 
             );

   PROCEDURE set_logging_level(
             p_id  IN VARCHAR2,
             p_lvl IN PLS_INTEGER 
             );

   PROCEDURE end_log(
             p_id IN VARCHAR2 
             );

END log_pkg;
/

CREATE OR REPLACE PACKAGE BODY log_pkg AS

   -------------------------------------------------------------

   FUNCTION start_log(
            p_id  IN VARCHAR2,
            p_lvl IN PLS_INTEGER DEFAULT 1 
            ) RETURN VARCHAR2 IS

      v_sid INTEGER;
      v_id  VARCHAR2(100);

   BEGIN

      SELECT sid INTO v_sid FROM v$mystat WHERE ROWNUM = 1;

      v_id := p_id||':'||v_sid;

      set_logging_level(v_id, p_lvl);

      RETURN v_id;

   END start_log;

   -------------------------------------------------------------

   FUNCTION logging_level(
            p_id IN VARCHAR2
            ) RETURN INTEGER IS

      v_lvl INTEGER;

   BEGIN

      SELECT logging_level INTO v_lvl
      FROM   log_cfg
      WHERE  id = p_id;

      RETURN v_lvl;

   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         RAISE_APPLICATION_ERROR(
            -20000, 'ID not found in logging context'
            );
   END logging_level;

   -------------------------------------------------------------

   PROCEDURE log(
             p_id  IN VARCHAR2,
             p_msg IN VARCHAR2,
             p_lvl IN PLS_INTEGER DEFAULT 1 
             ) IS
   BEGIN
      IF logging_level(p_id) >= p_lvl THEN
         DBMS_OUTPUT.PUT_LINE(
            TO_CHAR(SYSDATE,'HH24:MI:SS ": "') || p_msg
            );
      END IF;
   END log;

   -------------------------------------------------------------

   PROCEDURE set_logging_level( 
             p_id  IN VARCHAR2,
             p_lvl IN PLS_INTEGER 
             ) IS

      PRAGMA AUTONOMOUS_TRANSACTION;

   BEGIN

      MERGE
         INTO log_cfg tgt
         USING (
                SELECT p_id  AS id
                 ,     p_lvl AS logging_level
                FROM   dual
               ) src
         ON (src.id = tgt.id)
      WHEN MATCHED 
      THEN
         UPDATE
         SET    tgt.logging_level = src.logging_level
      WHEN NOT MATCHED
      THEN
         INSERT (tgt.id, tgt.logging_level)
         VALUES (src.id, src.logging_level);

      COMMIT;

   END set_logging_level;

   -------------------------------------------------------------

   PROCEDURE end_log(
             p_id  IN VARCHAR2
             ) IS
      PRAGMA AUTONOMOUS_TRANSACTION;
   BEGIN
      DELETE FROM log_cfg WHERE id = p_id;
      COMMIT;
   END end_log;

END log_pkg;
/

