
set echo on

CREATE OR REPLACE PACKAGE log_pkg AS

   PROCEDURE start_log( 
             p_id  IN VARCHAR2,
             p_lvl IN PLS_INTEGER DEFAULT 1 
             );

   PROCEDURE log( 
             p_id  IN VARCHAR2,
             p_msg IN VARCHAR2,
             p_lvl IN PLS_INTEGER DEFAULT 1 
             );

   PROCEDURE set_logging_level(
             p_id  IN VARCHAR2,
             p_lvl IN PLS_INTEGER 
             );

   PROCEDURE end_log(
             p_id IN VARCHAR2 
             );

END log_pkg;
/

CREATE OR REPLACE PACKAGE BODY log_pkg AS

   -------------------------------------------------------------

   PROCEDURE start_log(
             p_id  IN VARCHAR2,
             p_lvl IN PLS_INTEGER DEFAULT 1 
             ) IS
   BEGIN
      set_logging_level(p_id, p_lvl);
   END start_log;

   -------------------------------------------------------------

   FUNCTION logging_level(
            p_id IN VARCHAR2
            ) RETURN INTEGER IS
   BEGIN
      RETURN TO_NUMBER(SYS_CONTEXT('log_ctx', UPPER(p_id)));
   END logging_level;

   -------------------------------------------------------------

   PROCEDURE log(
             p_id  IN VARCHAR2,
             p_msg IN VARCHAR2,
             p_lvl IN PLS_INTEGER DEFAULT 1 
             ) IS
   BEGIN
      IF logging_level(p_id) >= p_lvl THEN
         DBMS_OUTPUT.PUT_LINE(
            TO_CHAR(SYSDATE,'HH24:MI:SS ": "') || p_msg
            );
      END IF;
   END log;

   -------------------------------------------------------------

   PROCEDURE set_logging_level( 
             p_id  IN VARCHAR2,
             p_lvl IN PLS_INTEGER 
             ) IS
   BEGIN
      DBMS_SESSION.SET_CONTEXT( 
         'log_ctx', UPPER(p_id), TO_CHAR(p_lvl)
         );
   END set_logging_level;

   -------------------------------------------------------------

   PROCEDURE end_log(
             p_id  IN VARCHAR2
             ) IS
   BEGIN
      DBMS_SESSION.CLEAR_CONTEXT('log_ctx', NULL, UPPER(p_id));
   END end_log;

END log_pkg;
/

